#!/bin/bash
#------------------------------------------------------------------------------
# Copyright (c) Perforce Software, Inc., 2007-2014. All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1  Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2.  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE
# SOFTWARE, INC. BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#------------------------------------------------------------------------------

# Intended to be run on a replica machine to sync replica from its corresponding master
# Like sync_replica.sh but for use in a regular crontab

source /p4/common/bin/backup_functions.sh

######### Start of Script ##########
LOG=$LOGS/sync_replica.log
check_vars

$P4BIN -u $P4USER -p ${SSL_PREFIX}${P4MASTER}:${P4MASTERPORTNUM} login < /p4/common/bin/adminpass > /dev/null 2>&1
JOURNALNUM=`$P4BIN -u $P4USER -p ${SSL_PREFIX}${P4MASTER}:${P4MASTERPORTNUM} counter journal`

if [[ "$JOURNALNUM" = "" ]]; then
   echo "Error:  Couldn't get journal number from master.  Aborting."
   exit 1
fi

# You must set up a public keypair using "ssh-keygen -t rsa" in order for this to work.
# You need to paste your CLIENT ~/.ssh/id_rsa.pub contents into the REMOTE ~/ssh/authorized_keys file. 
rsync -avz --delete ${OSUSER}@${P4MASTER}:$CHECKPOINTS/ $CHECKPOINTS > $LOG 2>&1
rsync_exit_code=$?

if [[ $rsync_exit_code -ne 0 ]]; then
   echo "Error: Failed to pull /p4/$P4INSTANCE/checkpoints from host $P4MASTER.  The rsync exit code was: $rsync_exit_code.  Aborting."
   exit 1
fi

#Unset P4NAME to allow p4d 2012.2 to recover checkpoint
unset P4NAME

rm -f /p4/$P4INSTANCE/offline_db/db.*
/p4/$P4INSTANCE/bin/p4d_$P4INSTANCE -r /p4/$P4INSTANCE/offline_db -jr -z /p4/$P4INSTANCE/checkpoints/p4_${P4INSTANCE}.ckp.${JOURNALNUM}.gz >> $LOG 2>&1

/p4/common/bin/p4master_run $P4INSTANCE /p4/${P4INSTANCE}/bin/p4d_${P4INSTANCE}_init stop >> $LOG 2>&1

# Sleep 30 seconds to give everything a chance to exit.
sleep 30

cd /p4/${P4INSTANCE}/root
rm -f save/db.* >> $LOG 2>&1
mv db.* save >> $LOG 2>&1
mv /p4/${P4INSTANCE}/offline_db/db.* . >> $LOG 2>&1
rm -f state >> $LOG 2>&1
rm -f rdb.lbr >> $LOG 2>&1
rm -f /p4/$P4INSTANCE/logs/log >> $LOG 2>&1
rm -f /p4/$P4INSTANCE/logs/journal >> $LOG 2>&1

$P4BIN -u $P4SERVICEUSER -p ${SSL_PREFIX}${P4MASTER}:${P4MASTERPORTNUM} login < /p4/common/bin/servicepass
/p4/common/bin/p4master_run $P4INSTANCE /p4/${P4INSTANCE}/bin/p4d_${P4INSTANCE}_init start >> $LOG 2>&1

# Recover the checkpoint again to keep the offline_db prepped for offline checkpoints in case the replica has to become the master.
/p4/$P4INSTANCE/bin/p4d_$P4INSTANCE -r /p4/$P4INSTANCE/offline_db -jr -z /p4/$P4INSTANCE/checkpoints/p4_${P4INSTANCE}.ckp.${JOURNALNUM}.gz >> $LOG 2>&1
